@@ -2,7 +2,7 @@ package Tk::Reindex;
 
 
 use vars qw($VERSION);
-$VERSION = '4.005'; # $Id: //depot/Tkutf8/TextList/Reindex.pm#4 $
+$VERSION = '4.006'; # $Id: //depot/Tkutf8/TextList/Reindex.pm#4 $
 
 use Tk;
 use base qw(Tk::Derived);
@@ -62,7 +62,7 @@ BEGIN
     {
       my($w)=shift;
       my(@iargs)=grep($_<=$#_,@{$argsub->(@_)});
-      my $iret=shift @iargs if @iargs && $iargs[0]==-1;
+      my $iret; $iret=shift @iargs if @iargs && $iargs[0]==-1;
       my(@args)=@_;
       @args[@iargs]=$w->Callback(-toindexcmd,@args[@iargs]);
       my(@ret)=$w->_callbase($sub,@args);
