@@ -1040,6 +1040,11 @@ BitmapInit(dispPtr)
         Tk_DefineBitmap(dummy, "warning", (char *) warning_bits,
 	        warning_width, warning_height);
 
+        Tk_DefineBitmap(dummy, "transparent", (char *) transparent_bits,
+	        transparent_width, transparent_height);
+        Tk_DefineBitmap(dummy, "Tk", (char *) Tkbmp_bits,
+	        Tkbmp_width, Tkbmp_height);
+
         TkpDefineNativeBitmaps();
         Tcl_DeleteInterp(dummy);
     }
