@@ -12,7 +12,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkPanedWindow.c,v 1.8 2002/08/08 01:43:31 hobbs Exp $
+ * RCS: @(#) $Id: tkPanedWindow.c,v 1.13.2.4 2003/08/19 21:00:13 jenglish Exp $
  */
 
 #include "tkPort.h"
@@ -203,8 +203,12 @@ static char *	ComputeSlotAddress _ANSI_ARGS_((char *recordPtr, int offset));
 static int	PanedWindowIdentifyCoords _ANSI_ARGS_((PanedWindow *pwPtr,
 			Tcl_Interp *interp, int x, int y));
 
+/*
+ * Sashes are between panes only, so there is one less sash than slaves
+ */
+
 #define ValidSashIndex(pwPtr, sash) \
-	(((sash) >= 0) && ((sash) < (pwPtr)->numSlaves))
+	(((sash) >= 0) && ((sash) < ((pwPtr)->numSlaves-1)))
 
 static Tk_GeomMgr panedWindowMgrType = {
     "panedwindow",		/* name */
@@ -389,10 +393,16 @@ Tk_PanedWindowObjCmd(clientData, interp, objc, objv)
     pwPtr->cursor	= None;
     pwPtr->sashCursor	= None;
 
+    /*
+     * Keep a hold of the associated tkwin until we destroy the widget,
+     * otherwise Tk might free it while we still need it.
+     */
+
+    Tcl_Preserve((ClientData) pwPtr->tkwin);
+
     if (Tk_InitOptions(interp, (char *) pwPtr, pwOpts->pwOptions,
 	    tkwin) != TCL_OK) {
 	Tk_DestroyWindow(pwPtr->tkwin);
-	ckfree((char *) pwPtr);
 	return TCL_ERROR;
     }
 
@@ -414,13 +424,20 @@ Tk_PanedWindowObjCmd(clientData, interp, objc, objv)
     }
 
     pwPtr->proxywin = Tk_CreateAnonymousWindow(interp, parent, (char *) NULL);
+    /*
+     * The proxy window has to be able to share GCs with the main
+     * panedwindow despite being children of windows with potentially
+     * different characteristics, and it looks better that way too.
+     * [Bug 702230]
+     */
+    Tk_SetWindowVisual(pwPtr->proxywin,
+	    Tk_Visual(tkwin), Tk_Depth(tkwin), Tk_Colormap(tkwin));
     Tk_CreateEventHandler(pwPtr->proxywin, ExposureMask, ProxyWindowEventProc,
 	    (ClientData) pwPtr);
 
     if (ConfigurePanedWindow(interp, pwPtr, objc - 2, objv + 2) != TCL_OK) {
-	Tk_DestroyWindow(pwPtr->tkwin);
 	Tk_DestroyWindow(pwPtr->proxywin);
-	ckfree((char *) pwPtr);
+	Tk_DestroyWindow(pwPtr->tkwin);
 	return TCL_ERROR;
     }
 
@@ -1207,7 +1224,7 @@ PanedWindowWorldChanged(instanceData)
 
     /*
      * Allocated a graphics context for drawing the paned window widget
-     * elements (background, sashes, etc.).
+     * elements (background, sashes, etc.) and set the window background.
      */
 
     gcValues.background = Tk_3DBorderColor(pwPtr->background)->pixel;
@@ -1216,6 +1233,7 @@ PanedWindowWorldChanged(instanceData)
 	Tk_FreeGC(pwPtr->display, pwPtr->gc);
     }
     pwPtr->gc = newGC;
+    Tk_SetWindowBackground(pwPtr->tkwin, gcValues.background);
 
     /*
      * Issue geometry size requests to Tk.
@@ -1309,8 +1327,8 @@ PanedWindowCmdDeletedProc(clientData)
      */
 
     if (!(pwPtr->flags & WIDGET_DELETED)) {
-	Tk_DestroyWindow(pwPtr->tkwin);
 	Tk_DestroyWindow(pwPtr->proxywin);
+	Tk_DestroyWindow(pwPtr->tkwin);
     }
 }
 
@@ -1355,8 +1373,7 @@ DisplayPanedWindow(clientData)
      */
 
     pixmap = Tk_GetPixmap(Tk_Display(tkwin), Tk_WindowId(tkwin),
-	    Tk_Width(tkwin), Tk_Height(tkwin),
-	    DefaultDepthOfScreen(Tk_Screen(tkwin)));
+	    Tk_Width(tkwin), Tk_Height(tkwin), Tk_Depth(tkwin));
 
     /*
      * Redraw the widget's background and border.
@@ -1478,6 +1495,7 @@ DestroyPanedWindow(pwPtr)
      */
 
     Tk_FreeConfigOptions((char *) pwPtr, pwPtr->optionTable, pwPtr->tkwin);
+    Tcl_Release((ClientData) pwPtr->tkwin);
     pwPtr->tkwin = NULL;
 
     Tcl_EventuallyFree((ClientData) pwPtr, TCL_DYNAMIC);
@@ -1509,14 +1527,21 @@ PanedWindowReqProc(clientData, tkwin)
     Tk_Window tkwin;		/* Other Tk-related information
 				 * about the window. */
 {
-    Slave *panePtr = (Slave *) clientData;
-    PanedWindow *pwPtr = (PanedWindow *) (panePtr->masterPtr);
+    Slave *slavePtr = (Slave *) clientData;
+    PanedWindow *pwPtr = (PanedWindow *) (slavePtr->masterPtr);
     if (Tk_IsMapped(pwPtr->tkwin)) {
 	if (!(pwPtr->flags & RESIZE_PENDING)) {
 	    pwPtr->flags |= RESIZE_PENDING;
 	    Tcl_DoWhenIdle(ArrangePanes, (ClientData) pwPtr);
 	}
     } else {
+	int doubleBw = 2 * Tk_Changes(slavePtr->tkwin)->border_width;
+	if (slavePtr->width <= 0) {
+	    slavePtr->paneWidth = Tk_ReqWidth(slavePtr->tkwin) + doubleBw;
+	}
+	if (slavePtr->height <= 0) {
+	    slavePtr->paneHeight = Tk_ReqHeight(slavePtr->tkwin) + doubleBw;
+	}
 	ComputeGeometry(pwPtr);
     }
 }
@@ -1627,10 +1652,12 @@ ArrangePanes(clientData)
 	if (pwPtr->orient == ORIENT_HORIZONTAL) {
 	    paneWidth = slavePtr->paneWidth;
 	    if (i == pwPtr->numSlaves - 1 && Tk_IsMapped(pwPtr->tkwin)) {
-		if (Tk_Width(pwPtr->tkwin) > Tk_ReqWidth(pwPtr->tkwin)) {
+		if (Tk_Width(pwPtr->tkwin) != Tk_ReqWidth(pwPtr->tkwin)) {
 		    paneWidth += Tk_Width(pwPtr->tkwin) -
-			Tk_ReqWidth(pwPtr->tkwin) -
-			Tk_InternalBorderWidth(pwPtr->tkwin);
+			Tk_ReqWidth(pwPtr->tkwin);
+		    if (paneWidth < 0) {
+			paneWidth = 0;
+		    }
 		}
 	    }
 	    paneHeight = Tk_Height(pwPtr->tkwin) - (2 * slavePtr->pady) -
@@ -1638,10 +1665,12 @@ ArrangePanes(clientData)
 	} else {
 	    paneHeight = slavePtr->paneHeight;
 	    if (i == pwPtr->numSlaves - 1 && Tk_IsMapped(pwPtr->tkwin)) {
-		if (Tk_Height(pwPtr->tkwin) > Tk_ReqHeight(pwPtr->tkwin)) {
+		if (Tk_Height(pwPtr->tkwin) != Tk_ReqHeight(pwPtr->tkwin)) {
 		    paneHeight += Tk_Height(pwPtr->tkwin) -
-			Tk_ReqHeight(pwPtr->tkwin) -
-			Tk_InternalBorderWidth(pwPtr->tkwin);
+			Tk_ReqHeight(pwPtr->tkwin);
+		    if (paneHeight < 0) {
+			paneHeight = 0;
+		    }
 		}
 	    }
 	    paneWidth = Tk_Width(pwPtr->tkwin) - (2 * slavePtr->padx) -
@@ -2236,7 +2265,6 @@ MoveSash(pwPtr, sash, diff)
 	 * Growing the pane, at the expense of panes to the right.
 	 */
 
-
 	/*
 	 * First check that moving the sash the requested distance will not
 	 * leave it off the screen.  If necessary, clip the requested diff
@@ -2427,8 +2455,7 @@ DisplayProxyWindow(clientData)
      */
 
     pixmap = Tk_GetPixmap(Tk_Display(tkwin), Tk_WindowId(tkwin),
-	    Tk_Width(tkwin), Tk_Height(tkwin),
-	    DefaultDepthOfScreen(Tk_Screen(tkwin)));
+	    Tk_Width(tkwin), Tk_Height(tkwin), Tk_Depth(tkwin));
 
     /*
      * Redraw the widget's background and border.
