@@ -9,10 +9,11 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclWinChan.c,v 1.24 2002/07/08 10:08:58 vincentdarley Exp $
+ * RCS: @(#) $Id: tclWinChan.c,v 1.30 2003/01/26 05:59:38 mdejong Exp $
  */
 
 #include "tclWinInt.h"
+#include "tclIO.h"
 
 /*
  * State flags used in the info structures below.
@@ -121,10 +122,14 @@ static Tcl_ChannelType fileChannelType = {
     FileWideSeekProc,		/* Wide seek proc. */
 };
 
-#ifdef HAVE_NO_SEH
-static void *ESP;
-static void *EBP;
-#endif /* HAVE_NO_SEH */
+#if defined(HAVE_NO_SEH) && defined(TCL_MEM_DEBUG)
+static void *INITIAL_ESP,
+            *INITIAL_EBP,
+            *INITIAL_HANDLER,
+            *RESTORED_ESP,
+            *RESTORED_EBP,
+            *RESTORED_HANDLER;
+#endif /* HAVE_NO_SEH && TCL_MEM_DEBUG */
 
 
 /*
@@ -385,9 +390,7 @@ FileCloseProc(instanceData, interp)
     Tcl_Interp *interp;		/* Not used. */
 {
     FileInfo *fileInfoPtr = (FileInfo *) instanceData;
-    FileInfo **nextPtrPtr;
     int errorCode = 0;
-    ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
 
     /*
      * Remove the file from the watch list.
@@ -397,11 +400,11 @@ FileCloseProc(instanceData, interp)
 
     /*
      * Don't close the Win32 handle if the handle is a standard channel
-     * during the exit process.  Otherwise, one thread may kill the stdio
-     * of another.
+     * during the thread exit process.  Otherwise, one thread may kill
+     * the stdio of another.
      */
 
-    if (!TclInExit()
+    if (!TclInThreadExit()
 	    || ((GetStdHandle(STD_INPUT_HANDLE) != fileInfoPtr->handle)
 		&& (GetStdHandle(STD_OUTPUT_HANDLE) != fileInfoPtr->handle)
 		&& (GetStdHandle(STD_ERROR_HANDLE) != fileInfoPtr->handle))) {
@@ -410,13 +413,7 @@ FileCloseProc(instanceData, interp)
 	    errorCode = errno;
 	}
     }
-    for (nextPtrPtr = &(tsdPtr->firstFilePtr); (*nextPtrPtr) != NULL;
-	 nextPtrPtr = &((*nextPtrPtr)->nextPtr)) {
-	if ((*nextPtrPtr) == fileInfoPtr) {
-	    (*nextPtrPtr) = fileInfoPtr->nextPtr;
-	    break;
-	}
-    }
+
     ckfree((char *)fileInfoPtr);
     return errorCode;
 }
@@ -467,7 +464,7 @@ FileSeekProc(instanceData, offset, mode, errorCodePtr)
     oldPos = SetFilePointer(infoPtr->handle, (LONG)0, &oldPosHigh,
 	    FILE_CURRENT);
     if (oldPos == INVALID_SET_FILE_POINTER) {
-	int winError = GetLastError();
+	DWORD winError = GetLastError();
 	if (winError != NO_ERROR) {
 	    TclWinConvertError(winError);
 	    *errorCodePtr = errno;
@@ -479,7 +476,7 @@ FileSeekProc(instanceData, offset, mode, errorCodePtr)
     newPos = SetFilePointer(infoPtr->handle, (LONG) offset, &newPosHigh,
 			    moveMethod);
     if (newPos == INVALID_SET_FILE_POINTER) {
-	int winError = GetLastError();
+	DWORD winError = GetLastError();
 	if (winError != NO_ERROR) {
 	    TclWinConvertError(winError);
 	    *errorCodePtr = errno;
@@ -540,7 +537,7 @@ FileWideSeekProc(instanceData, offset, mode, errorCodePtr)
     newPos = SetFilePointer(infoPtr->handle, (LONG) offset, &newPosHigh,
 			    moveMethod);
     if (newPos == INVALID_SET_FILE_POINTER) {
-	int winError = GetLastError();
+	DWORD winError = GetLastError();
 	if (winError != NO_ERROR) {
 	    TclWinConvertError(winError);
 	    *errorCodePtr = errno;
@@ -1034,7 +1031,7 @@ Tcl_MakeFileChannel(rawHandle, mode)
 		GetCurrentProcess(), &dupedHandle, 0, FALSE,
 		DUPLICATE_SAME_ACCESS);
 
-	if (result != 0) {
+	if (result == 0) {
 	    /*
 	     * Unable to make a duplicate. It's definately invalid at this
 	     * point.
@@ -1049,34 +1046,59 @@ Tcl_MakeFileChannel(rawHandle, mode)
 	 */
 
 #ifdef HAVE_NO_SEH
+# ifdef TCL_MEM_DEBUG
         __asm__ __volatile__ (
-                "movl  %esp, _ESP" "\n\t"
-                "movl  %ebp, _EBP");
-
-        __asm__ __volatile__ (
-                "pushl $__except_makefilechannel_handler" "\n\t"
-                "pushl %fs:0" "\n\t"
-                "mov   %esp, %fs:0");
+            "movl %%esp,  %0" "\n\t"
+            "movl %%ebp,  %1" "\n\t"
+            "movl %%fs:0, %2" "\n\t"
+            : "=m"(INITIAL_ESP),
+              "=m"(INITIAL_EBP),
+              "=r"(INITIAL_HANDLER) );
+# endif /* TCL_MEM_DEBUG */
 
         result = 0;
+
+        __asm__ __volatile__ (
+            "pushl %ebp" "\n\t"
+            "pushl $__except_makefilechannel_handler" "\n\t"
+            "pushl %fs:0" "\n\t"
+            "movl  %esp, %fs:0");
 #else
 	__try {
 #endif /* HAVE_NO_SEH */
 	    CloseHandle(dupedHandle);
 #ifdef HAVE_NO_SEH
         __asm__ __volatile__ (
-                "jmp   makefilechannel_pop" "\n"
-                "makefilechannel_reentry:" "\n\t"
-                "movl  _ESP, %esp" "\n\t"
-                "movl  _EBP, %ebp");
+            "jmp  makefilechannel_pop" "\n"
+        "makefilechannel_reentry:" "\n\t"
+            "movl %%fs:0, %%eax" "\n\t"
+            "movl 0x8(%%eax), %%esp" "\n\t"
+            "movl 0x8(%%esp), %%ebp" "\n"
+            "movl $1, %0" "\n"
+        "makefilechannel_pop:" "\n\t"
+            "movl (%%esp), %%eax" "\n\t"
+            "movl %%eax, %%fs:0" "\n\t"
+            "add  $12, %%esp" "\n\t"
+            : "=m"(result)
+            :
+            : "%eax");
 
-        result = 1;  /* True when exception was raised */
+# ifdef TCL_MEM_DEBUG
+    __asm__ __volatile__ (
+            "movl  %%esp,  %0" "\n\t"
+            "movl  %%ebp,  %1" "\n\t"
+            "movl  %%fs:0, %2" "\n\t"
+            : "=m"(RESTORED_ESP),
+              "=m"(RESTORED_EBP),
+              "=r"(RESTORED_HANDLER) );
 
-        __asm__ __volatile__ (
-                "makefilechannel_pop:" "\n\t"
-                "mov   (%esp), %eax" "\n\t"
-                "mov   %eax, %fs:0" "\n\t"
-                "add   $8, %esp");
+    if (INITIAL_ESP != RESTORED_ESP)
+        panic("ESP restored incorrectly");
+    if (INITIAL_EBP != RESTORED_EBP)
+        panic("EBP restored incorrectly");
+    if (INITIAL_HANDLER != RESTORED_HANDLER)
+        panic("HANDLER restored incorrectly");
+# endif /* TCL_MEM_DEBUG */
 
         if (result)
             return NULL;
@@ -1116,6 +1138,8 @@ _except_makefilechannel_handler(
 {
     __asm__ __volatile__ (
             "jmp makefilechannel_reentry");
+    /* Nuke compiler warning about unused static function */
+    _except_makefilechannel_handler(NULL, NULL, NULL, NULL);
     return 0; /* Function does not return */
 }
 #endif
@@ -1314,3 +1338,93 @@ TclWinFlushDirtyChannels ()
 	}
     }
 }
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TclpCutFileChannel --
+ *
+ *	Remove any thread local refs to this channel. See
+ *	Tcl_CutChannel for more info.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Changes thread local list of valid channels.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TclpCutFileChannel(chan)
+    Tcl_Channel chan;			/* The channel being removed. Must
+                                         * not be referenced in any
+                                         * interpreter. */
+{
+    ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
+    Channel *chanPtr = (Channel *) chan;
+    FileInfo *infoPtr;
+    FileInfo **nextPtrPtr;
+    int removed = 0;
+
+    if (chanPtr->typePtr != &fileChannelType)
+        return;
+
+    infoPtr = (FileInfo *) chanPtr->instanceData;
+
+    for (nextPtrPtr = &(tsdPtr->firstFilePtr); (*nextPtrPtr) != NULL;
+	 nextPtrPtr = &((*nextPtrPtr)->nextPtr)) {
+	if ((*nextPtrPtr) == infoPtr) {
+	    (*nextPtrPtr) = infoPtr->nextPtr;
+	    removed = 1;
+	    break;
+	}
+    }
+
+    /*
+     * This could happen if the channel was created in one thread
+     * and then moved to another without updating the thread
+     * local data in each thread.
+     */
+
+    if (!removed)
+        panic("file info ptr not on thread channel list");
+
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TclpSpliceFileChannel --
+ *
+ *	Insert thread local ref for this channel.
+ *	Tcl_SpliceChannel for more info.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Changes thread local list of valid channels.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TclpSpliceFileChannel(chan)
+    Tcl_Channel chan;			/* The channel being removed. Must
+                                         * not be referenced in any
+                                         * interpreter. */
+{
+    ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
+    Channel *chanPtr = (Channel *) chan;
+    FileInfo *infoPtr;
+
+    if (chanPtr->typePtr != &fileChannelType)
+        return;
+
+    infoPtr = (FileInfo *) chanPtr->instanceData;
+
+    infoPtr->nextPtr = tsdPtr->firstFilePtr;
+    tsdPtr->firstFilePtr = infoPtr;
+}
