@@ -38,10 +38,10 @@
 #endif
 
 FORWARD int   DItemParseProc _ANSI_ARGS_((ClientData clientData,
-		Tcl_Interp *interp, Tk_Window tkwin, Arg value,
+		Tcl_Interp *interp, Tk_Window tkwin, Tcl_Obj * value,
 		char *widRec, int offset));
 
-FORWARD Arg   DItemPrintProc _ANSI_ARGS_((
+FORWARD Tcl_Obj *   DItemPrintProc _ANSI_ARGS_((
 		ClientData clientData, Tk_Window tkwin, char *widRec,
 		int offset, Tcl_FreeProc **freeProcPtr));
 
@@ -508,7 +508,7 @@ DItemParseProc(clientData, interp, tkwin, value, widRec,offset)
     ClientData clientData;
     Tcl_Interp *interp;
     Tk_Window tkwin;
-    Arg value;
+    Tcl_Obj * value;
     char *widRec;
     int offset;
 {
@@ -528,7 +528,7 @@ DItemParseProc(clientData, interp, tkwin, value, widRec,offset)
     return TCL_OK;
 }
 
-LINKAGE Arg
+LINKAGE Tcl_Obj *
 DItemPrintProc(clientData, tkwin, widRec,offset, freeProcPtr)
     ClientData clientData;
     Tk_Window tkwin;
