@@ -161,7 +161,7 @@ TkSelGetSelection(interp, tkwin, selection, target, proc, clientData)
      * Pass the data off to the selection procedure.
      */
 
-    result = (*proc)(clientData, interp, Tcl_DStringValue(&ds),
+    result = (*proc)(clientData, interp, (long *) Tcl_DStringValue(&ds),
                      Tcl_DStringLength(&ds),8,target,tkwin);
     Tcl_DStringFree(&ds);
     CloseClipboard();
@@ -193,7 +193,11 @@ error:
  *----------------------------------------------------------------------
  */
 
-
+Window
+XGetSelectionOwner(Display *display, Atom selection)
+{
+ return None;
+}
 
 void
 XSetSelectionOwner(display, selection, owner, time)
